/* 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * *****
 *
 * Notes:
 *
 *    Since we can't know ahead of time where Opera will be installed, we Kiboze the
 *    Start Menu tree.  If the shortcut has been renamed (i.e. it is no longer
 *    titled "Opera"), we're out of luck.  The only option at that
 *    point would be to open and dereference every symlink we find, and see if the
 *    target leaf name is "opera.exe".  Not doing that at the moment, since
 *    it just seems insane.
 */

var operaviewMenuItems = new Array("operaview-do-view", "operaview-do-viewlink");
var userPrograms = "Progs";
var allUserPrograms = "CmPrgs";
var applicationData = "AppData";

var gOperaViewBundle;

function operaviewContext() {
  if(gContextMenu) {
    for(var i=0; i<operaviewMenuItems.length; i++) {
      var menuitem = document.getElementById(operaviewMenuItems[i]);
      if(menuitem && (i == 0))
        menuitem.hidden = (gContextMenu.isTextSelected || gContextMenu.onLink || gContextMenu.onImage || gContextMenu.onTextInput );
      else if (menuitem)
      {
         menuitem.hidden = ! gContextMenu.onLink;
      }
    }
  }
}

function operaView() {
  if(gContextMenu) {
     var href = gBrowser.currentURI.spec;

     operaViewLaunch("Opera.lnk", href);
  }
}


function operaViewLink() {
  if(gContextMenu) {
  
    var href;
    
	  if (gContextMenu && gContextMenu.getLinkURL)
  		href = gContextMenu.getLinkURL();
	  else if (gContextMenu && gContextMenu.linkURL)
  		href = gContextMenu.linkURL();

      operaViewLaunch("Opera.lnk", href);
  }
}



// attempt to grab the real path of a predefined directory
//
function tryDir(dsp, key)
{
   try
   {
      var nif = dsp.get(key, Components.interfaces.nsIFile);
      return(nif.path);
   }
   catch (ar)
   {
      return("");
   }
}


function searchPath(path, fname)
{
   var result = null;

   try
   {
      var f = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);

      f.initWithPath(path);

      if (f.exists() && f.isDirectory())
      {
         var entries = f.directoryEntries;

         while (entries.hasMoreElements())
         {
            var ent = entries.getNext().QueryInterface(Components.interfaces.nsIFile);

            if (ent.isDirectory())
            {
               result = searchPath(ent.path, fname);

               if (result)
               {
                  break;
               }
            }
            else if (ent.isSymlink())
            {
               if (ent.leafName.toLowerCase() == fname.toLowerCase())
               {
                  result = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
                  result.followLinks = true;
                  result.initWithPath(ent.path);

                  if (result.target == "")
                  {
                     result = null;
                  }
                  else
                  {
                     break;
                  }
               }
            }
            else if (ent.isFile())
            {
               if (ent.leafName.toLowerCase() == fname.toLowerCase())
               {
                  result = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
                  result.initWithPath(ent.path);
                  break;
               }
            }
         }
      }
   }
   catch (ar)
   {
      return(null);
   }

   return( result );
}



function operaViewLaunch (path,argumentstext)
{
    var cantMessage = "can't find";

    try
    {
        cantMessage = gOperaViewBundle.getString("operaview.cantFindExplorer");
    }
    catch(e)
    {
        alert(e);
    }

  var prefservice = Components.classes["@mozilla.org/preferences-service;1"].
                getService(Components.interfaces.nsIPrefService);   

  var prefs = prefservice.getBranch("");

   try{
      if(path=="") return false;

      var operaloc = null;

      if (prefs.getPrefType("operaview.operaapp") == prefs.PREF_STRING)
      {
        operaloc = prefs.getCharPref("operaview.operaapp");
      }

      var   natTarget = null;
      var   usePath = null;

      if (operaloc != null)
      {
         natTarget = operaloc;
      }
      else
      {
         var dsprops = Components.classes['@mozilla.org/file/directory_service;1'].getService(Components.interfaces.nsIProperties);

         usePath = tryDir(dsprops, userPrograms);    // try user-specific program menu first

         var file = null;

         if (usePath != "")
         {
            file = searchPath(usePath, path);
         }

         if (! file)
         {
            usePath = tryDir(dsprops, allUserPrograms);   // no joy?  try "all users" program menu

            if (usePath != "")
            {
               file = searchPath(usePath, path);
            }
         }

         // (almost) last ditch -- find the windows directory
         // assume that the main Program Files directory is on the same drive
         // look in there, under Program Files\Opera, for opera.exe
         //
         if (! file)
         {
            var usePath = tryDir(dsprops, "WinD");

            if ((usePath != "") && (usePath.charAt(1) == ":"))
            {
               usePath = usePath.substr(0, 2) + "\\program files\\opera";

               file = searchPath(usePath, "opera.exe");
            }
         }

				 // the very last ditch -- try to run just 'opera'         
         if (! file)
         {
            file = searchPath(".", "opera");
         }

         if ((! file) || (! file.exists()))
         {
            alert(cantMessage);
            return false;
         }

         natTarget = file.target;
      }

      var targetFile = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);

      targetFile.initWithPath(natTarget);

      if (! targetFile.exists())
      {
         alert(cantMessage);
         return(false);
      }

      usePath = targetFile.target;

      var process = Components.classes['@mozilla.org/process/util;1'].getService(Components.interfaces.nsIProcess);
      process.init(targetFile);
      var arguments= [] ;

      arguments.push(argumentstext);

      process.run(false, arguments, arguments.length, {});
      return true;

   }catch(e){ 
      alert(e);return false;
   }

   return false;		// avoid JavaScript Error.
}


function operaviewInit() {
  var menu = document.getElementById("contentAreaContextMenu");
  menu.addEventListener("popupshowing",operaviewContext,false);

  gOperaViewBundle = document.getElementById("bundle_operaview");

  if (! gOperaViewBundle)
  {
      alert("no bundle");
  }
  
  try {
			const nsISupportsString = Components.interfaces.nsISupportsString;
    
			var pref = Components.classes["@mozilla.org/preferences-service;1"]
									.getService(Components.interfaces.nsIPrefService)
									.getBranch("extensions.{87f54a61-c9b3-4138-a38a-33c31770bb9e}.");
    
			var str = Components.classes["@mozilla.org/supports-string;1"]
									.createInstance(nsISupportsString);
    
			str.data = gOperaViewBundle.getString("operaview.description");
            
			pref.setComplexValue("description", nsISupportsString, str);
	} catch (err) {}

}

function setOperaviewOptions()
{
		var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                               .getService(Components.interfaces.nsIPrefService);
    
    if (document.getElementById('operaloc').value)
    {
				var prefs = prefService.getBranch("");
				prefs.setCharPref("operaview.operaapp", document.getElementById('operaloc').value);
    }
    
    if (document.getElementById('showicons') != null)
    {
			var prefs = prefService.getBranch("");
			prefs.setBoolPref("operaview.showicons", document.getElementById('showicons').checked);
    }
    
    window.close();
}

function pickOpera()
{
    var picker = Components.classes["@mozilla.org/filepicker;1"].getService(Components.interfaces.nsIFilePicker);

    picker.init(window, "OperaView", 0);
    picker.appendFilters(64);
    
    if (picker.show() == 0)
    {
        document.getElementById('operaloc').value = picker.file.target;
    }
}


function init()
{
    var prefservice = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);   
    
    var prefs = prefservice.getBranch("");

    var operaloc = null;

    if (prefs.getPrefType("operaview.operaapp") == prefs.PREF_STRING)
    {
      operaloc = prefs.getCharPref("operaview.operaapp");
    }

    if ((operaloc != null) && (operaloc.length > 0)) 
    {
        document.getElementById('operaloc').value = operaloc;
    }
    
    var opera_showicons = null;
    
    if (prefs.getPrefType("operaview.showicons") == prefs.PREF_BOOL)
    {
      opera_showicons = prefs.getBoolPref("operaview.showicons");
    }

    if (opera_showicons != null) 
    {
        document.getElementById('showicons').checked = opera_showicons;
    }

    prefs = null;
}

// do the init on load
window.addEventListener("load", operaviewInit, false); 